using System;
using System.Collections.Generic;
using System.IO;
using Autofac;
using FitConnect;
using FluentAssertions;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Primitives;
using MockContainer;
using Moq;
using NUnit.Framework;

namespace IntegrationTests;

[TestFixture]
public class CallbackTest {
    [SetUp]
    public void Setup() {
        var memoryStream = new MemoryStream();
        var streamWriter = new StreamWriter(memoryStream);

        streamWriter.WriteLine(
            "{\"type\":\"https://schema.fitko.de/fit-connect/submission-api/callbacks/new-submissions\",\"submissionIds\":[\"f39ab143-d91a-474a-b69f-b00f1a1873c2\"]}");
        streamWriter.Flush();
        memoryStream.Position = 0;


        // Request = new DefaultHttpRequest(new DefaultHttpContext()) {
        //     Body = new StreamBody(memoryStream)
        // };

        var headers = new HeaderDictionary(new Dictionary<string, StringValues> {
            { "callback-timestamp", "4102354800" }, {
                "callback-authentication",
                "56740c29a40e6031d9db0cbd1a1b620e2c561a12307eb61ee8ceab22c7c44a65e753cd92f4cbe4386acbfd03064d64f64915fc03f342c818a4f43eb97192b5c0"
            }
        });


        var mock =
            new Mock<HttpRequest>();
        mock.Setup(w => w.ContentType).Returns("application/json");
        mock.Setup(w => w.Headers).Returns(headers);
        mock.Setup(w => w.Method).Returns("POST");
        mock.Setup(w => w.Body).Returns(memoryStream);

        _request = mock.Object;

        _callbackSecret = Container.Create().Resolve<IFitConnectSettings>().CallbackSecret;
    }

    private HttpRequest _request = null!;
    private string _callbackSecret = "";


    [Test]
    public void ValidRequest_WithSingeValues() {
        // Arrange

        //Act
        var authentication = FitConnect.Subscriber.VerifyCallback(_callbackSecret, 4102354800,
            "{\"type\":\"https://schema.fitko.de/fit-connect/submission-api/callbacks/new-submissions\",\"submissionIds\":[\"f39ab143-d91a-474a-b69f-b00f1a1873c2\"]}"
        );

        // Assert
        authentication.Should()
            .Be(
                "56740c29a40e6031d9db0cbd1a1b620e2c561a12307eb61ee8ceab22c7c44a65e753cd92f4cbe4386acbfd03064d64f64915fc03f342c818a4f43eb97192b5c0");
    }

    [Test]
    public void ValidRequest() {
        // Assert
        FitConnect.Subscriber.VerifyCallback(_callbackSecret, _request).Should().Be(true);
    }

    [Test]
    public void RequestAge_Fails() {
        // Arrange
        _request.Headers["callback-timestamp"] = "1641066653";

        // Atc
        // Assert
        Assert.Throws<ArgumentException>(() => {
                FitConnect.Subscriber.VerifyCallback(_callbackSecret, _request);
            })
            ?.Message.Should().Be("Request is too old");
    }

    [Test]
    public void RequestAuthentication_Fails() {
        // Arrange
        _request.Headers["callback-authentication"] =
            "898cd0edb70c08e5b32aa8a18cbbc8ff6b3078c51af6d011ff4e32e470c746234fc4314821fe5185264b029e962bd37de33f3b9fc5f1a93c40ce6672845e90df";

        // Atc
        // Assert
        Assert.Throws<ArgumentException>(() => {
                FitConnect.Subscriber.VerifyCallback(_callbackSecret, _request);
            })
            ?.Message.Should().Be("Verified request does not match authentication");
    }
}
